/*
 *	Copyright (C) 2001,2004 Jan Bobrowski <jb@wizard.ae.krakow.pl>
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	version 2 as published by the Free Software Foundation.
 */

#include <unistd.h>
#include "toico.h"

void oom()
{
	errx(2, "Out of memory");
}

char *image_name;

void freadn(void *p, int n, FILE *f)
{
	int v = fread(p, 1, n, f);
	if(v < n)
		errx(1, "%s: Unexpected end of file", image_name);
}

void alloc_im(struct img *i)
{
	i->im = (u32*)malloc(i->w * i->h * sizeof(*i->im));
	if(!i->im) oom();
}

struct img img[16];
int nimg = 0;
struct opt opt = {ALPHA};

static void newinput(FILE *f, char *n)
{
	struct img *i = &img[nimg];
	u8 sig[SIGN_SIZE];

	image_name = n;
	freadn(sig, sizeof sig, f);
	i->name = n;
#ifdef WITH_PNG
	if(ispng(sig))
		readpng(i, f, sig);
	else
#endif
	if(isgif(sig))
		readgif(i, f, sig);
	else
#ifdef WITH_TIFF
	if(istiff(sig))
		readtiff(i, f, sig);
	else
#endif
	if(isbmp(sig))
		readbmp(i, f, sig);
	else if(isxpm(sig))
		readxpm(i, f, sig);
	else
		errx(1, "%s: Unsupported file format", n);
	nimg++;
}

void usage()
{
	printf(
		"usage: toico [-vx] [-k RRGGBB] [-o icon] pic1 [pic2...]\n"
		"       toico [-?V]\n"
		"	-o  output to a file instead of stdout\n"
		"	-x  allow full alpha channel (32-bit icons)\n"
		"	-k  transparency key for BMP files (usually magenta: FF00FF)\n"
		"	-v  be verbose\n"
		"	-V  print toico version\n"
		"formats supported:"
#ifdef WITH_PNG
		" PNG"
#endif
		" GIF"
#ifdef WITH_TIFF
		" TIFF"
#endif
		" BMP XPM\n"
	);
	exit(1);
}

int main(int argc, char **argv)
{
	int n;
	FILE *of;
	char *oname = 0;
	extern char *optarg;
	extern int optind;

	while((n=getopt(argc, argv, "o:xk:v?hV"))!=EOF) switch(n) {
		case 'o': oname = optarg; break;
		case 'x': opt.xp = 1; break;
		case 'k': {
			char *p;
			opt.alpha_key = strtoul(optarg, &p, 16);
			if(*p) errx(1, "`%s' should be in RRGGBB format", optarg);
		} break;
		case 'v': opt.verbose = 1; break;
		case 'h':
		case '?': usage();
		case 'V':
			printf("toico "STR(VERSION)"\n"
				"Copyright 2004 Jan Bobrowski\n"
				"License: GPL (freeware)\n");
			exit(0);
	}

	if(optind >= argc && isatty(fileno(stdin)))
		usage();

	if(opt.verbose && !transparent(opt.alpha_key))
		printf("key: %06X\n", opt.alpha_key);
	if(!oname) {
		of = stdout;
		if(isatty(fileno(of)))
			errx(1, "Output not specified");
	}

	for(n=optind; n<argc; n++) {
		if(nimg == elemof(img)) errx(1, "Too many images");
		if(argv[n][0] == '-' && argv[n][1] == 0)
			newinput(stdin, "<stdin>");
		else {
			FILE *f = fopen(argv[n], "rb");
			if(!f) err(1, "%s", argv[n]);
			newinput(f, argv[n]);
			fclose(f);
		}
	}
	if(!nimg)
		newinput(stdin, "<stdin>");

	if(oname) {
		of = fopen(oname, "wb");
		if(!of) err(1, "%s", oname);
	}
	writeico(img, nimg, of);
	return 0;
}
