#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>

#define elemof(T) (sizeof T/sizeof*T)
#define endof(T) (T+elemof(T))
#define _STR(S) #S
#define STR(S) _STR(S)

#ifndef NO_ERRWARN
#include <err.h>
#else
void err(int,char*,...);
void errx(int,char*,...);
void warn(char*,...);
void warnx(char*,...);
#endif

typedef unsigned char u8;
typedef unsigned short u16;
typedef unsigned long u32;

#if defined(BYTE_ORDER)
#if BYTE_ORDER == BIG_ENDIAN
#define BE_ORDER
#endif
#else
#ifdef sparc
#define BE_ORDER
#endif
#endif

#ifndef __linux__
static inline u32 bswap_32(u32 v) {v = v>>16 | v<<16; return v<<8&0xFF00FF00 | v>>8&0x00FF00FF;}
static inline u16 bswap_16(u16 v) {return v<<8 | v>>8;}
#else
#include <byteswap.h>
#endif

#ifndef BE_ORDER
static inline u16 le16(u16 v) {return v;}
static inline u32 le32(u32 v) {return v;}
static inline u16 be16(u16 v) {return bswap_16(v);}
static inline u32 be32(u32 v) {return bswap_32(v);}
#else
static inline u16 le16(u16 v) {return bswap_16(v);}
static inline u32 le32(u32 v) {return bswap_32(v);}
static inline u16 be16(u16 v) {return v;}
static inline u32 be32(u32 v) {return v;}
#endif

static inline u16 get16le(void *p) {return le16(*(u16*)p);}
static inline u32 get32le(void *p) {return le32(*(u32*)p);}

#define SIGN_SIZE 8

struct opt {
	u32 alpha_key;	/* for BMP */
	int verbose:1;
	int xp:1;	/* generate semitransparent icons */
};

extern struct opt opt;

struct img {
	int w, h;
	u32 *im;
	int depth;
	int npal;
	u32 pal[256];
	char *name;
};

#define IMGEND(I) ((I)->im+(I)->w*(I)->h)
#define IMGAT(I,X,Y) ((I)->im+(I)->w*(Y)+(X))

void oom();
void freadn(void *p, int n, FILE *f);
void alloc_im(struct img *img);
int ispng(u8*);
int isgif(u8*);
int istiff(u8*);
int isbmp(u8*);
int isxpm(u8*);
void readpng(struct img*, FILE*, u8[SIGN_SIZE]);
void readgif(struct img*, FILE*, u8[SIGN_SIZE]);
void readtiff(struct img*, FILE*, u8[SIGN_SIZE]);
void readxpm(struct img*, FILE*, u8[SIGN_SIZE]);
void readbmp(struct img*, FILE*, u8[SIGN_SIZE]);
void writeico(struct img*, int, FILE*);

#define ALPHA 0xFF000000u
static inline transparent(u32 c) {return (c&ALPHA) == ALPHA;}
